#!/tvbin/tivosh
#
# Standalone script to build the Series->Program cache
# Called from update_series_cache{} in index.itcl
#
# Reads the NSL list in MFS and build a disk file of "Seriesfsid {Programfsid *}"
# (disc file is subsequently read by load_series_cache{} in index.itcl
#
# July 2012 [spitfires]
#
#

# Filename for the generated cache
set filename "/var/tmp/series_cache"


proc appendlog text {
	set fd [open "/var/log/tivoweb.log" a]; puts $fd $text; close $fd
}

proc timer {action {var ""}} {
	if {$var != ""} {
		upvar 1 $var timer
	} else {
		upvar #0 etime timer
	} 
	switch $action {
		"start" { set timer [clock clicks] }
		"stop"	{ return [format {%.2f} [expr ([clock clicks]-$timer)/1e6]] }
		"disp"	{ return "[bodytext_start]<br /><small>Elapsed Time: [format {%.2f} [expr ([clock clicks]-$timer)/1e6]] secs</small>[bodytext_end]" }
	}
}

proc set_tivo_version {} {
	set db [dbopen]
	RetryTransaction {
		set swsystem [db $db open /SwSystem/ACTIVE]
		set tivoswversion [dbobj $swsystem get Name]
		set ::version [lindex [split $tivoswversion .] 0]
	}
}
set_tivo_version

proc make_series_cache fn {
  if {$::version >= 3} {
    set nowshowingdir "/Recording/NowShowingByClassic"
  } else {
    set nowshowingdir "/Recording/NowShowing"
  }

  file delete -force $fn	;# delete the file to stop the out-of-date one being reloaded
	
	RetryTransaction {
		set cache_ns_cnt [mfs scancount $nowshowingdir]
  }
	appendlog " >>cache_ns_cnt=$cache_ns_cnt"
  if {$cache_ns_cnt == -1} {
    return
  }

  set db [dbopen]
	
  ForeachMfsFile fsid name type $nowshowingdir "" { 
    RetryTransaction {
			set rec [db $db openid $fsid]
			set showing [dbobj $rec get Showing]
			set program [dbobj $showing get Program]
			set seriesfsid [dbobj $program gettarget Series]
			regsub {/-1$} $seriesfsid {} seriesfsid
			if {$seriesfsid != ""} {
				if {[info exists cache_series_rec($seriesfsid)]} {
					lappend cache_series_rec($seriesfsid) $fsid
				} else {
					set cache_series_rec($seriesfsid) $fsid
				}
			}
		}
  }
	
	appendlog " >>size=[array size cache_series_rec] "
	# puts cache_series_rec=[array get cache_series_rec]

	set fd [open $fn.tmp w]
	foreach {key value} [array get cache_series_rec] { 
		puts $fd "$key $value"
	}
	close $fd
	file copy -force $fn.tmp $fn
	file delete -force $fn.tmp
}


appendlog "Building Series->Program cache"		;# (redirecting output with ">>&" doesn't work when backgrounded)
timer start
make_series_cache $filename
appendlog "Build complete : [timer stop] secs"
